<?php
session_start();
if (!isset($_SESSION['user'])) {
    header("Location: ../login.php");
    exit;
}

// 引入数据库配置文件
require_once '../config.php';

// 数据库连接
$conn = get_db_connection();

// 获取参数
$product_id = isset($_GET['product_id']) ? intval($_GET['product_id']) : 0;
$specified_warehouse_id = isset($_GET['warehouse_id']) ? intval($_GET['warehouse_id']) : 0;
$view = isset($_GET['view']) ? $_GET['view'] : '';

// 验证产品ID
if ($product_id <= 0) {
    die('无效的商品ID');
}

// 获取商品信息
$productStmt = $conn->prepare("SELECT id, product_code, product_name, unit, multi_unit FROM products WHERE id = ?");
$productStmt->bind_param("i", $product_id);
$productStmt->execute();
$productResult = $productStmt->get_result();

if ($productResult->num_rows === 0) {
    die('商品不存在');
}

$product = $productResult->fetch_assoc();

// 获取商品单位信息
$baseUnitName = $product['unit'];
$unitStmt = $conn->prepare("SELECT unit_name FROM product_units WHERE product_id = ? AND unit_type = 'base' LIMIT 1");
$unitStmt->bind_param("i", $product_id);
$unitStmt->execute();
$unitResult = $unitStmt->get_result();
if ($unitResult->num_rows > 0) {
    $baseUnitName = $unitResult->fetch_assoc()['unit_name'];
}

// 获取仓库列表
$warehouseQuery = "SELECT id, name FROM warehouses";
if ($specified_warehouse_id > 0) {
    $warehouseQuery .= " WHERE id = " . $specified_warehouse_id;
}
$warehouseQuery .= " ORDER BY name";
$warehouseResult = $conn->query($warehouseQuery);
$warehouses = [];
while ($row = $warehouseResult->fetch_assoc()) {
    $warehouses[] = $row;
}

// 获取当前库存数据
$stockData = [];
$stockQuery = "
    SELECT 
        w.id AS warehouse_id, 
        w.name AS warehouse_name, 
        COALESCE(s.base_quantity, 0) AS stock_quantity,
        s.id AS stock_id
    FROM warehouses w
    LEFT JOIN inventory_stock s ON w.id = s.warehouse_id AND s.product_id = ?
";
if ($specified_warehouse_id > 0) {
    $stockQuery .= " WHERE w.id = " . $specified_warehouse_id;
}
$stockQuery .= " ORDER BY w.name";

$stockStmt = $conn->prepare($stockQuery);
$stockStmt->bind_param("i", $product_id);
$stockStmt->execute();
$stockResult = $stockStmt->get_result();

while ($row = $stockResult->fetch_assoc()) {
    $stockData[$row['warehouse_id']] = [
        'warehouse_id' => $row['warehouse_id'],
        'warehouse_name' => $row['warehouse_name'],
        'stock_quantity' => $row['stock_quantity'],
        'stock_id' => $row['stock_id']
    ];
}

// 处理表单提交
$message = '';
if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    // 开始事务
    $conn->begin_transaction();

    try {
        foreach ($_POST as $key => $value) {
            if (strpos($key, 'quantity_') === 0) {
                $warehouseId = intval(substr($key, 9));
                $quantity = floatval($value);

                // 检查是否已有库存记录
                $checkStmt = $conn->prepare("SELECT id FROM inventory_stock WHERE product_id = ? AND warehouse_id = ?");
                $checkStmt->bind_param("ii", $product_id, $warehouseId);
                $checkStmt->execute();
                $checkResult = $checkStmt->get_result();

                if ($checkResult->num_rows > 0) {
                    // 更新现有库存
                    $stockId = $checkResult->fetch_assoc()['id'];
                    $updateStmt = $conn->prepare("UPDATE inventory_stock SET base_quantity = ?, updated_by = ?, last_updated = NOW() WHERE id = ?");
                    $userId = $_SESSION['user']['id'];
                    $updateStmt->bind_param("dii", $quantity, $userId, $stockId);
                    $updateStmt->execute();
                } else if ($quantity > 0) {
                    // 只有数量大于0时才新增库存记录
                    $insertStmt = $conn->prepare("INSERT INTO inventory_stock (product_id, warehouse_id, base_quantity, updated_by) VALUES (?, ?, ?, ?)");
                    $userId = $_SESSION['user']['id'];
                    $insertStmt->bind_param("iidi", $product_id, $warehouseId, $quantity, $userId);
                    $insertStmt->execute();
                }
            }
        }

        // 提交事务
        $conn->commit();
        $message = '✅ 库存更新成功！';

        // 清除Redis缓存
        $redis = get_redis_instance();
        if ($redis->isAvailable()) {
            // 清除库存相关缓存
            $redis->delete("inventory_stock:product:*");
            $redis->delete("inventory_stock:warehouse:*");
            error_log("Redis cache cleared after inventory update");
        }

        // 重定向回库存列表页，保留视图参数
        $redirectUrl = "inventory_stock.php?message=" . urlencode($message);
        if ($view === 'warehouse') {
            $redirectUrl .= "&view=warehouse";
            // 如果是按仓库视图且指定了仓库ID，也保留该参数
            if ($specified_warehouse_id > 0) {
                $redirectUrl .= "&warehouse_id=" . $specified_warehouse_id;
            }
        }
        header("Location: " . $redirectUrl . "&refresh=1");
        exit;
    } catch (Exception $e) {
        // 回滚事务
        $conn->rollback();
        $message = '❌ 更新失败：' . $e->getMessage();
    }
}

// 关闭数据库连接
$conn->close();
?>
<!DOCTYPE html>
<html lang="zh">

<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>编辑库存 - <?= htmlspecialchars($product['product_name']) ?></title>
    <link rel="stylesheet" href="style.css">
    <style>
        .product-info {
            background-color: #f8f9fa;
            padding: 15px;
            border-radius: 5px;
            margin-bottom: 20px;
        }

        .product-info p {
            margin: 5px 0;
        }

        .product-info strong {
            display: inline-block;
            width: 100px;
        }

        /* 统一表格样式 */
        .inventory-table {
            border-collapse: collapse;
            width: 100%;
            margin-bottom: 20px;
        }

        .inventory-table th {
            background-color: #f8f9fa;
            padding: 10px;
            text-align: left;
            border-bottom: 2px solid #ddd;
        }

        .inventory-table td {
            padding: 12px 10px;
            border-bottom: 1px solid #eee;
            vertical-align: middle;
        }

        .inventory-table .quantity-input,
        .inventory-table .unit-select {
            width: 100%;
            height: 36px;
            padding: 8px 10px;
            border: 1px solid #ddd;
            border-radius: 4px;
            box-sizing: border-box;
            font-family: inherit;
            font-size: 14px;
        }

        .inventory-table .quantity-input:focus,
        .inventory-table .unit-select:focus {
            border-color: #2b9348;
            outline: none;
            box-shadow: 0 0 0 2px rgba(43, 147, 72, 0.25);
        }
    </style>
</head>

<body>
    <div class="header">
        <h2>编辑库存</h2>
        <a href="inventory_stock.php<?= $view === 'warehouse' ? '?view=warehouse' . ($specified_warehouse_id > 0 ? '&warehouse_id=' . $specified_warehouse_id : '') : '' ?>" class="btn btn-secondary">返回库存列表</a>
    </div>

    <!-- 消息显示 -->
    <?php if (!empty($message)): ?>
        <div id="message-container" class="message" style="display: flex;">
            <span id="message-text"><?= $message ?></span>
            <button onclick="document.getElementById('message-container').style.display='none'">×</button>
        </div>
    <?php endif; ?>

    <div class="container">
        <div class="product-info">
            <p><strong>商品编号：</strong><?= htmlspecialchars($product['product_code']) ?></p>
            <p><strong>商品名称：</strong><?= htmlspecialchars($product['product_name']) ?></p>
            <p><strong>基本单位：</strong><?= htmlspecialchars($baseUnitName) ?></p>
        </div>

        <form method="post" class="inventory-form">
            <input type="hidden" id="product_id" name="product_id" value="<?= $product_id ?>">
            <input type="hidden" name="view" value="<?= htmlspecialchars($view) ?>">

            <table class="inventory-table">
                <thead>
                    <tr>
                        <th style="width: 25%;">仓库名称</th>
                        <th style="width: 25%;">单位选择</th>
                        <th style="width: 25%;">当前库存</th>
                        <th style="width: 25%;">新库存数量</th>
                    </tr>
                </thead>
                <tbody>
                    <?php foreach ($warehouses as $warehouse): ?>
                        <?php
                        $currentStock = isset($stockData[$warehouse['id']]) ? $stockData[$warehouse['id']]['stock_quantity'] : 0;
                        ?>
                        <tr>
                            <td><?= htmlspecialchars($warehouse['name']) ?></td>
                            <td>
                                <select name="unit_type_<?= $warehouse['id'] ?>"
                                    id="unit_type_<?= $warehouse['id'] ?>"
                                    class="unit-select"
                                    data-warehouse-id="<?= $warehouse['id'] ?>"
                                    data-base-quantity="<?= $currentStock ?>">
                                    <option value="base" data-ratio="1"><?= htmlspecialchars($baseUnitName) ?></option>
                                    <!-- 副单位将通过JavaScript动态添加 -->
                                </select>
                            </td>
                            <td>
                                <div class="current-stock" id="current_stock_<?= $warehouse['id'] ?>">
                                    <?= floor($currentStock) ?> <?= $baseUnitName ?>
                                </div>
                            </td>
                            <td>
                                <input type="number"
                                    name="quantity_<?= $warehouse['id'] ?>"
                                    id="quantity_<?= $warehouse['id'] ?>"
                                    value=""
                                    min="0"
                                    step="1"
                                    class="quantity-input"
                                    data-current="<?= floor($currentStock) ?>"
                                    data-base-quantity="<?= $currentStock ?>">
                                <input type="hidden" name="base_quantity_<?= $warehouse['id'] ?>"
                                    id="base_quantity_<?= $warehouse['id'] ?>"
                                    value="<?= $currentStock ?>">
                            </td>
                        </tr>
                    <?php endforeach; ?>
                </tbody>
            </table>

            <div class="btn-group">
                <button type="submit" class="btn btn-success">保存库存</button>
                <button type="button" class="btn btn-secondary" onclick="window.location.href='inventory_stock.php<?= $view === 'warehouse' ? '?view=warehouse' . ($specified_warehouse_id > 0 ? '&warehouse_id=' . $specified_warehouse_id : '') : '' ?>'">取消</button>
            </div>
        </form>
    </div>

    <script>
        // 自动关闭消息提示
        setTimeout(() => {
            const messageBox = document.getElementById('message-container');
            if (messageBox) {
                messageBox.style.display = 'none';
            }
        }, 3000);

        // 存储单位转换数据
        let unitData = [];
        let baseUnit = "";

        // 获取商品单位
        document.addEventListener('DOMContentLoaded', function() {
            const productId = document.getElementById('product_id').value;
            if (productId) {
                fetch(`get_product_units.php?product_id=${productId}`)
                    .then(response => response.json())
                    .then(data => {
                        unitData = data;

                        // 查找基本单位
                        const baseUnitObj = data.find(unit => unit.unit_type === 'base');
                        if (baseUnitObj) {
                            baseUnit = baseUnitObj.unit_name;
                        }

                        // 添加副单位到所有选择框
                        document.querySelectorAll('.unit-select').forEach(select => {
                            data.forEach(unit => {
                                if (unit.unit_type === 'sub') {
                                    const option = document.createElement('option');
                                    option.value = unit.unit_name;
                                    option.textContent = unit.unit_name;
                                    option.dataset.ratio = unit.ratio;
                                    select.appendChild(option);
                                }
                            });
                        });
                    })
                    .catch(error => {
                        console.error('获取商品单位失败:', error);
                    });
            }
        });

        // 单位转换处理
        document.querySelectorAll('.unit-select').forEach(select => {
            select.addEventListener('change', function() {
                const warehouseId = this.dataset.warehouseId;
                const baseQuantity = parseFloat(this.dataset.baseQuantity) || 0;
                const selectedOption = this.options[this.selectedIndex];
                const ratio = parseFloat(selectedOption.dataset.ratio) || 1;
                const unitName = selectedOption.value;

                // 转换当前库存显示
                const convertedCurrentStock = convertQuantity(baseQuantity, ratio);
                const currentStockText = document.getElementById(`current_stock_${warehouseId}`);
                currentStockText.textContent = `${convertedCurrentStock} ${unitName}`;

                // 转换并更新新库存输入框
                const quantityInput = document.getElementById(`quantity_${warehouseId}`);
                const baseQuantityInput = document.getElementById(`base_quantity_${warehouseId}`);

                // 当前输入框的值也要转换
                quantityInput.value = convertedCurrentStock;
                quantityInput.dataset.ratio = ratio;

                // 设置步进值为合适的精度
                quantityInput.step = ratio < 1 ? "0.01" : "1";
            });
        });

        // 更新输入框时，更新隐藏的基本单位数量
        document.querySelectorAll('.quantity-input').forEach(input => {
            input.addEventListener('input', function() {
                const warehouseId = this.id.split('_')[1];
                const select = document.getElementById(`unit_type_${warehouseId}`);
                const selectedOption = select.options[select.selectedIndex];
                const ratio = parseFloat(selectedOption.dataset.ratio) || 1;
                const value = parseFloat(this.value) || 0;

                // 转换回基本单位数量
                const baseQuantity = value * ratio;
                document.getElementById(`base_quantity_${warehouseId}`).value = baseQuantity;
            });
        });

        // 数量转换函数
        function convertQuantity(baseQuantity, ratio) {
            ratio = ratio || 1;
            const convertedQuantity = baseQuantity / ratio;
            // 如果比率小于1，保留两位小数；否则向下取整
            return ratio < 1 ? Math.floor(convertedQuantity * 100) / 100 : Math.floor(convertedQuantity);
        }

        // 在提交表单前，确保使用基本单位的数量提交
        document.querySelector('.inventory-form').addEventListener('submit', function(e) {
            document.querySelectorAll('.quantity-input').forEach(input => {
                const warehouseId = input.id.split('_')[1];
                const baseQuantityInput = document.getElementById(`base_quantity_${warehouseId}`);
                input.value = baseQuantityInput.value;
            });
        });
    </script>
</body>

</html>