<?php
session_start();
if (!isset($_SESSION['user'])) {
    header('Content-Type: application/json');
    echo json_encode(['error' => '未登录']);
    exit;
}

// 引入数据库配置文件
require_once '../config.php';

// 确保有输出缓冲区可供清理
ob_start();

// 设置错误处理，防止PHP错误破坏JSON输出
set_error_handler(function ($errno, $errstr, $errfile, $errline) {
    return true;
});

try {
    // 验证仓库ID
    if (!isset($_GET['warehouse_id']) || !is_numeric($_GET['warehouse_id'])) {
        // 确保在返回JSON前清理缓冲区
        if (ob_get_length()) {
            ob_clean();
        }
        header('Content-Type: application/json');
        echo json_encode(['error' => '无效的仓库ID']);
        exit;
    }

    $warehouse_id = intval($_GET['warehouse_id']);

    // 数据库连接
    $conn = get_db_connection();

    // 查询该仓库有库存的商品
    $sql = "SELECT 
                p.id, 
                p.product_code, 
                p.product_name, 
                p.unit, 
                p.multi_unit, 
                s.base_quantity as stock_quantity
            FROM 
                products p
            JOIN 
                inventory_stock s ON p.id = s.product_id
            WHERE 
                s.warehouse_id = ? 
                AND s.base_quantity > 0
            ORDER BY 
                p.id DESC";

    $stmt = $conn->prepare($sql);
    $stmt->bind_param("i", $warehouse_id);
    $stmt->execute();
    $result = $stmt->get_result();

    $products = [];
    $multiUnitCount = 0;
    while ($row = $result->fetch_assoc()) {
        // 将商品基本信息添加到结果集
        $product = $row;

        // 如果是多单位商品，则一并获取其所有单位信息
        if ($row['multi_unit'] == 1) {
            $multiUnitCount++;

            $unitsSql = "SELECT id, product_id, unit_type, unit_name, ratio 
                        FROM product_units 
                        WHERE product_id = ? AND unit_type = 'sub'
                        ORDER BY id";
            $unitsStmt = $conn->prepare($unitsSql);
            $unitsStmt->bind_param("i", $row['id']);
            $unitsStmt->execute();
            $unitsResult = $unitsStmt->get_result();

            $product['units'] = [];
            while ($unitRow = $unitsResult->fetch_assoc()) {
                // 确保ratio是浮点数
                $unitRow['ratio'] = floatval($unitRow['ratio']);
                $product['units'][] = $unitRow;
            }

            if (empty($product['units'])) {
                // 警告: 商品ID={$row['id']}是多单位商品，但没有查询到单位信息
            }

            $unitsStmt->close();
        }

        $products[] = $product;
    }

    // 关闭数据库连接
    $stmt->close();
    $conn->close();

    // 确保没有之前的输出
    if (ob_get_length()) {
        ob_clean();
    }

    // 返回JSON格式的商品数据
    header('Content-Type: application/json');
    echo json_encode($products);
    exit;
} catch (Exception $e) {
    // 清除之前的任何输出
    if (ob_get_length()) {
        ob_clean();
    }

    header('Content-Type: application/json');
    echo json_encode(['error' => '服务器错误', 'message' => $e->getMessage()]);
    exit;
}
